"""Gentoo logo"""

from archey.colors import Colors


COLORS = [Colors.MAGENTA_BRIGHT, Colors.WHITE_BRIGHT]

LOGO = [
    """{c[0]}          -/oyddmdhs+:.             """,
    """{c[0]}      -o{c[1]}dNMMMMMMMMNNmhy+{c[0]}-`          """,
    """{c[0]}    -y{c[1]}NMMMMMMMMMMMNNNmmdhy{c[0]}+-        """,
    """{c[0]}  `o{c[1]}mMMMMMMMMMMMMNmdmmmmddhhy{c[0]}/`     """,
    """{c[0]}  om{c[1]}MMMMMMMMMMMN{c[0]}hhyyyo{c[1]}hmdddhhhd{c[0]}o`   """,
    """{c[0]} .y{c[1]}dMMMMMMMMMMd{c[0]}hs++so/s{c[1]}mdddhhhhdm{c[0]}+` """,
    """{c[0]}  oy{c[1]}hdmNMMMMMMMN{c[0]}dyooy{c[1]}dmddddhhhhyhN{c[0]}d.""",
    """{c[0]}   :o{c[1]}yhhdNNMMMMMMMNNNmmdddhhhhhyym{c[0]}Mh""",
    """{c[0]}     .:{c[1]}+sydNMMMMMNNNmmmdddhhhhhhmM{c[0]}my""",
    """{c[0]}        /m{c[1]}MMMMMMNNNmmmdddhhhhhmMNh{c[0]}s:""",
    """{c[0]}     `o{c[1]}NMMMMMMMNNNmmmddddhhdmMNhs{c[0]}+` """,
    """{c[0]}   `s{c[1]}NMMMMMMMMNNNmmmdddddmNMmhs{c[0]}/.   """,
    """{c[0]}  /N{c[1]}MMMMMMMMNNNNmmmdddmNMNdso{c[0]}:`     """,
    """{c[0]} +M{c[1]}MMMMMMNNNNNmmmmdmNMNdso{c[0]}/-        """,
    """{c[0]} yM{c[1]}MNNNNNNNmmmmmNNMmhs+/{c[0]}-`          """,
    """{c[0]} /h{c[1]}MMNNNNNNNNMNdhs++/{c[0]}-`             """,
    """{c[0]} `/{c[1]}ohdmmddhys+++/:{c[0]}.`                """,
    """{c[0]} `-//////:--.                       """
]
