"""Elementary OS logo"""

from archey.colors import Colors


COLORS = [Colors.WHITE_BRIGHT]

LOGO = [
    """{c[0]}          eeeeeeeeeeeeeeeee         """,
    """{c[0]}       eeeeeeeeeeeeeeeeeeeeeee      """,
    """{c[0]}     eeeee  eeeeeeeeeeee   eeeee    """,
    """{c[0]}   eeee   eeeee       eee     eeee  """,
    """{c[0]}  eeee   eeee          eee     eeee """,
    """{c[0]} eee    eee            eee       eee""",
    """{c[0]} eee   eee            eee        eee""",
    """{c[0]} ee    eee           eeee       eeee""",
    """{c[0]} ee    eee         eeeee      eeeeee""",
    """{c[0]} ee    eee       eeeee      eeeee ee""",
    """{c[0]} eee   eeee   eeeeee      eeeee  eee""",
    """{c[0]} eee    eeeeeeeeee     eeeeee    eee""",
    """{c[0]}  eeeeeeeeeeeeeeeeeeeeeeee    eeeee """,
    """{c[0]}   eeeeeeee eeeeeeeeeeee      eeee  """,
    """{c[0]}     eeeee                 eeeee    """,
    """{c[0]}       eeeeeee         eeeeeee      """,
    """{c[0]}          eeeeeeeeeeeeeeeee         """
]
